@@ -2,7 +2,7 @@
 /*
  * viking -- GPS Data and Topo Analyzer, Explorer, and Manager
  *
- * Copyright (C) 2012, Rob Norris <rw_norris@hotmail.com>
+ * Copyright (C) 2012-2015, Rob Norris <rw_norris@hotmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,8 +22,6 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
-#include <string.h>
-#include <limits.h>
 
 #include <glib/gprintf.h>
 #include <glib/gi18n.h>
@@ -35,8 +33,6 @@
 #include "gpx.h"
 #include "acquire.h"
 #include "osm-traces.h"
-#include "preferences.h"
-#include "curl_download.h"
 #include "datasource_gps.h"
 #include "bbox.h"
 
@@ -53,10 +49,10 @@ typedef struct {
 	VikViewport *vvp;
 } datasource_osm_my_traces_t;
 
-static gpointer datasource_osm_my_traces_init( );
+static gpointer datasource_osm_my_traces_init ( acq_vik_t *avt );
 static void datasource_osm_my_traces_add_setup_widgets ( GtkWidget *dialog, VikViewport *vvp, gpointer user_data );
-static void datasource_osm_my_traces_get_cmd_string ( gpointer user_data, gchar **args, gchar **extra, DownloadMapOptions *options );
-static gboolean datasource_osm_my_traces_process  ( VikTrwLayer *vtl, const gchar *cmd, const gchar *extra, BabelStatusFunc status_cb, acq_dialog_widgets_t *adw, DownloadMapOptions *options_unused );
+static void datasource_osm_my_traces_get_process_options ( gpointer user_data, ProcessOptions *po, DownloadMapOptions *options, const gchar *notused1, const gchar *notused2 );
+static gboolean datasource_osm_my_traces_process ( VikTrwLayer *vtl, ProcessOptions *process_options, BabelStatusFunc status_cb, acq_dialog_widgets_t *adw, DownloadMapOptions *options_unused );
 static void datasource_osm_my_traces_cleanup ( gpointer data );
 
 VikDataSourceInterface vik_datasource_osm_my_traces_interface = {
@@ -70,8 +66,8 @@ VikDataSourceInterface vik_datasource_osm_my_traces_interface = {
   (VikDataSourceInitFunc)		    datasource_osm_my_traces_init,
   (VikDataSourceCheckExistenceFunc)	NULL,
   (VikDataSourceAddSetupWidgetsFunc)datasource_osm_my_traces_add_setup_widgets,
-  (VikDataSourceGetCmdStringFunc)	datasource_osm_my_traces_get_cmd_string,
-  (VikDataSourceProcessFunc)		datasource_osm_my_traces_process,
+  (VikDataSourceGetProcessOptionsFunc) datasource_osm_my_traces_get_process_options,
+  (VikDataSourceProcessFunc)           datasource_osm_my_traces_process,
   (VikDataSourceProgressFunc)		NULL,
   (VikDataSourceAddProgressWidgetsFunc)	NULL,
   (VikDataSourceCleanupFunc)		datasource_osm_my_traces_cleanup,
@@ -84,7 +80,7 @@ VikDataSourceInterface vik_datasource_osm_my_traces_interface = {
   0
 };
 
-static gpointer datasource_osm_my_traces_init ( )
+static gpointer datasource_osm_my_traces_init ( acq_vik_t *avt )
 {
   datasource_osm_my_traces_t *data = g_malloc(sizeof(*data));
   // Reuse GPS functions
@@ -108,25 +104,28 @@ static void datasource_osm_my_traces_add_setup_widgets ( GtkWidget *dialog, VikV
 	user_label = gtk_label_new(_("Username:"));
 	data->user_entry = gtk_entry_new();
 
-	gtk_box_pack_start ( GTK_BOX(GTK_DIALOG(dialog)->vbox), user_label, FALSE, FALSE, 0 );
-	gtk_box_pack_start ( GTK_BOX(GTK_DIALOG(dialog)->vbox), data->user_entry, FALSE, FALSE, 0 );
+	gtk_box_pack_start ( GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), user_label, FALSE, FALSE, 0 );
+	gtk_box_pack_start ( GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), data->user_entry, FALSE, FALSE, 0 );
 	gtk_widget_set_tooltip_markup ( GTK_WIDGET(data->user_entry), _("The email or username used to login to OSM") );
 
 	password_label = gtk_label_new ( _("Password:") );
 	data->password_entry = gtk_entry_new ();
 
-	gtk_box_pack_start ( GTK_BOX(GTK_DIALOG(dialog)->vbox), password_label, FALSE, FALSE, 0 );
-	gtk_box_pack_start ( GTK_BOX(GTK_DIALOG(dialog)->vbox), data->password_entry, FALSE, FALSE, 0 );
 	gtk_widget_set_tooltip_markup ( GTK_WIDGET(data->password_entry), _("The password used to login to OSM") );
 
 	osm_login_widgets (data->user_entry, data->password_entry);
+
+	/* Packing all widgets */
+	GtkBox *box = GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog)));
+	gtk_box_pack_start ( box, password_label, FALSE, FALSE, 0 );
+	gtk_box_pack_start ( box, data->password_entry, FALSE, FALSE, 0 );
 	gtk_widget_show_all ( dialog );
 
 	/* Keep reference to viewport */
 	data->vvp = vvp;
 }
 
-static void datasource_osm_my_traces_get_cmd_string ( gpointer user_data, gchar **args, gchar **extra, DownloadMapOptions *options )
+static void datasource_osm_my_traces_get_process_options ( gpointer user_data, ProcessOptions *po, DownloadMapOptions *options, const gchar *notused1, const gchar *notused2 )
 {
 	datasource_osm_my_traces_t *data = (datasource_osm_my_traces_t*) user_data;
 
@@ -134,10 +133,8 @@ static void datasource_osm_my_traces_get_cmd_string ( gpointer user_data, gchar
 	osm_set_login ( gtk_entry_get_text ( GTK_ENTRY(data->user_entry) ),
 	                gtk_entry_get_text ( GTK_ENTRY(data->password_entry) ) );
 
-	// If going to use the values passed back into the process function parameters then these need to be set.
+	// If going to use the values passed back into the process function parameters then they need to be set.
 	// But ATM we aren't
-	*args = NULL;
-	*extra = NULL;
 	options = NULL;
 }
 
@@ -461,8 +458,8 @@ static GList *select_from_list (GtkWindow *parent, GList *list, const gchar *tit
 	gtk_scrolled_window_set_policy ( GTK_SCROLLED_WINDOW(scrolledwindow), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC );
 	gtk_container_add ( GTK_CONTAINER(scrolledwindow), view );
 
-	gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox), label, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox), scrolledwindow, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), label, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), scrolledwindow, TRUE, TRUE, 0);
 
 	// Ensure a reasonable number of items are shown, but let the width be automatically sized
 	gtk_widget_set_size_request ( dialog, -1, 400) ;
@@ -495,6 +492,7 @@ static GList *select_from_list (GtkWindow *parent, GList *list, const gchar *tit
 						}
 						list_runner = g_list_next ( list_runner );
 					}
+					g_free ( name );
 				}
 			}
 			while ( gtk_tree_model_iter_next ( GTK_TREE_MODEL(store), &iter ) );
@@ -518,7 +516,7 @@ static void none_found ( GtkWindow *gw )
 	gtk_window_set_title(GTK_WINDOW(dialog), _("GPS Traces"));
 
 	GtkWidget *search_label = gtk_label_new(_("None found!"));
-	gtk_box_pack_start ( GTK_BOX(GTK_DIALOG(dialog)->vbox), search_label, FALSE, FALSE, 5 );
+	gtk_box_pack_start ( GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), search_label, FALSE, FALSE, 5 );
 	gtk_dialog_set_default_response ( GTK_DIALOG(dialog), GTK_RESPONSE_ACCEPT );
 	gtk_widget_show_all(dialog);
 
@@ -560,7 +558,7 @@ static void set_in_current_view_property ( VikTrwLayer *vtl, datasource_osm_my_t
 	}
 }
 
-static gboolean datasource_osm_my_traces_process ( VikTrwLayer *vtl, const gchar *cmd, const gchar *extra, BabelStatusFunc status_cb, acq_dialog_widgets_t *adw, DownloadMapOptions *options_unused )
+static gboolean datasource_osm_my_traces_process ( VikTrwLayer *vtl, ProcessOptions *process_options, BabelStatusFunc status_cb, acq_dialog_widgets_t *adw, DownloadMapOptions *options_unused )
 {
 	//datasource_osm_my_traces_t *data = (datasource_osm_my_traces_t *)adw->user_data;
 
@@ -568,7 +566,12 @@ static gboolean datasource_osm_my_traces_process ( VikTrwLayer *vtl, const gchar
 
 	gchar *user_pass = osm_get_login();
 
-	DownloadMapOptions options = { FALSE, FALSE, NULL, 2, NULL, user_pass }; // Allow a couple of redirects
+	// Support .zip + bzip2 files directly
+	DownloadMapOptions options = { FALSE, FALSE, NULL, 2, NULL, user_pass, a_try_decompress_file }; // Allow a couple of redirects
+
+	gchar *tmpname = a_download_uri_to_tmp_file ( DS_OSM_TRACES_GPX_FILES, &options );
+	if ( !tmpname )
+		return FALSE;
 
 	xml_data *xd = g_malloc ( sizeof (xml_data) );
 	//xd->xpath = g_string_new ( "" );
@@ -577,22 +580,24 @@ static gboolean datasource_osm_my_traces_process ( VikTrwLayer *vtl, const gchar
 	xd->current_gpx_meta_data = new_gpx_meta_data_t();
 	xd->list_of_gpx_meta_data = NULL;
 
-	gchar *tmpname = a_download_uri_to_tmp_file ( DS_OSM_TRACES_GPX_FILES, &options );
 	result = read_gpx_files_metadata_xml ( tmpname, xd );
 	// Test already downloaded metadata file: eg:
 	//result = read_gpx_files_metadata_xml ( "/tmp/viking-download.GI47PW", xd );
 
 	if ( tmpname ) {
-		g_remove ( tmpname );
+		(void)util_remove ( tmpname );
 		g_free ( tmpname );
 	}
 
-	if ( ! result )
+	if ( ! result ) {
+		g_free ( xd );
 		return FALSE;
+	}
 
 	if ( g_list_length ( xd->list_of_gpx_meta_data ) == 0 ) {
 		if (!vik_datasource_osm_my_traces_interface.is_thread)
 			none_found ( GTK_WINDOW(adw->vw) );
+		g_free ( xd );
 		return FALSE;
 	}
 
@@ -604,6 +609,10 @@ static gboolean datasource_osm_my_traces_process ( VikTrwLayer *vtl, const gchar
 	GList *selected = select_from_list ( GTK_WINDOW(adw->vw), xd->list_of_gpx_meta_data, "Select GPS Traces", "Select the GPS traces you want to add." );
     if (vik_datasource_osm_my_traces_interface.is_thread) gdk_threads_leave();
 
+	// If non thread - show program is 'doing something...'
+	if ( !vik_datasource_osm_my_traces_interface.is_thread )
+		vik_window_set_busy_cursor ( adw->vw );
+
 	// If passed in on an existing layer - we will create everything into that.
 	//  thus with many differing gpx's - this will combine all waypoints into this single layer!
 	// Hence the preference is to create multiple layers
@@ -622,7 +631,7 @@ static gboolean datasource_osm_my_traces_process ( VikTrwLayer *vtl, const gchar
 
 		if ( create_new_layer ) {
 			// Have data but no layer - so create one
-			vtlX = VIK_TRW_LAYER ( vik_layer_create ( VIK_LAYER_TRW, vik_window_viewport(adw->vw), NULL, FALSE ) );
+			vtlX = VIK_TRW_LAYER ( vik_layer_create ( VIK_LAYER_TRW, adw->vvp, FALSE ) );
 			if ( ((gpx_meta_data_t*)selected_iterator->data)->name )
 				vik_layer_rename ( VIK_LAYER ( vtlX ), ((gpx_meta_data_t*)selected_iterator->data)->name );
 			else
@@ -634,19 +643,26 @@ static gboolean datasource_osm_my_traces_process ( VikTrwLayer *vtl, const gchar
 		if ( gpx_id ) {
 			gchar *url = g_strdup_printf ( DS_OSM_TRACES_GPX_URL_FMT, gpx_id );
 
-			result = a_babel_convert_from_url ( vtlX, url, "gpx", status_cb, adw, &options );
+			// NB download type is GPX (or a compressed version)
+			ProcessOptions my_po = *process_options;
+			my_po.url = url;
+			result = a_babel_convert_from ( vtlX, &my_po, status_cb, adw, &options );
 			// TODO investigate using a progress bar:
 			// http://developer.gnome.org/gtk/2.24/GtkProgressBar.html
 
 			got_something = got_something || result;
-			// TODO feedback to UI to inform which traces failed
-			if ( !result )
-				g_warning ( _("Unable to get trace: %s"), url );
+			if ( !result ) {
+				// Report errors to the status bar
+				gchar* msg = g_strdup_printf ( _("Unable to get trace: %s"), url );
+				vik_window_statusbar_update ( adw->vw, msg, VIK_STATUSBAR_INFO );
+				g_free (msg);
+			}
+			g_free ( url );
 		}
 
 		if ( result ) {
 			// Can use the layer
-			vik_aggregate_layer_add_layer ( vik_layers_panel_get_top_layer (adw->vlp), VIK_LAYER(vtlX) );
+			vik_aggregate_layer_add_layer ( vik_layers_panel_get_top_layer (adw->vlp), VIK_LAYER(vtlX), TRUE );
 			// Move to area of the track
 			vik_layer_post_read ( VIK_LAYER(vtlX), vik_window_viewport(adw->vw), TRUE );
 			vik_trw_layer_auto_set_view ( vtlX, vik_window_viewport(adw->vw) );
@@ -681,6 +697,9 @@ static gboolean datasource_osm_my_traces_process ( VikTrwLayer *vtl, const gchar
 		// Process was cancelled but need to return that it proceeded as expected
 		result = TRUE;
 
+	if ( !vik_datasource_osm_my_traces_interface.is_thread )
+		vik_window_clear_busy_cursor ( adw->vw );
+
 	return result;
 }
 
