@@ -2,7 +2,7 @@
  * viking -- GPS Data and Topo Analyzer, Explorer, and Manager
  *
  * Copyright (C) 2003-2005, Evan Battaglia <gtoevan@gmx.net>
- * Copyright (C) 2012, Rob Norris <rw_norris@hotmail.com>
+ * Copyright (C) 2012-2013, Rob Norris <rw_norris@hotmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,7 +46,6 @@ static void a_gpspoint_write_track ( const gpointer id, const VikTrack *t, FILE
 static void a_gpspoint_write_trackpoint ( VikTrackpoint *tp, TP_write_info_type *write_info );
 static void a_gpspoint_write_waypoint ( const gpointer id, const VikWaypoint *wp, FILE *f );
 
-
 /* outline for file gpspoint.c
 
 reading file:
@@ -157,7 +156,7 @@ static gchar *deslashndup ( const gchar *str, guint16 len )
  * No obvious way to test for a 'gpspoint' file,
  *  thus set a flag if any actual tag found during processing of the file
  */
-gboolean a_gpspoint_read_file(VikTrwLayer *trw, FILE *f ) {
+gboolean a_gpspoint_read_file(VikTrwLayer *trw, FILE *f, const gchar *dirpath ) {
   VikCoordMode coord_mode = vik_trw_layer_get_coord_mode ( trw );
   gchar *tag_start, *tag_end;
   g_assert ( f != NULL && trw != NULL );
@@ -190,7 +189,7 @@ gboolean a_gpspoint_read_file(VikTrwLayer *trw, FILE *f ) {
       /* my addition: find first non-whitespace character. if the null, skip line. */
       while (*tag_start != '\0' && isspace(*tag_start))
         tag_start++;
-      if (tag_start == '\0')
+      if (*tag_start == '\0')
         break;
 
       if (*tag_start == '#')
@@ -209,7 +208,9 @@ gboolean a_gpspoint_read_file(VikTrwLayer *trw, FILE *f ) {
           inside_quote = !inside_quote;
       }
 
-      gpspoint_process_tag ( tag_start, tag_end - tag_start );
+      // Won't have super massively long strings, so potential truncation in cast to gint is acceptable.
+      gint len = (gint)(tag_end - tag_start);
+      gpspoint_process_tag ( tag_start, len );
 
       if (*tag_end == '\0' )
         break;
@@ -237,8 +238,19 @@ gboolean a_gpspoint_read_file(VikTrwLayer *trw, FILE *f ) {
       if ( line_description )
         vik_waypoint_set_description ( wp, line_description );
 
-      if ( line_image )
-        vik_waypoint_set_image ( wp, line_image );
+      if ( line_image ) {
+        // Ensure the filename is absolute
+        if ( g_path_is_absolute ( line_image ) )
+          vik_waypoint_set_image ( wp, line_image );
+        else {
+          // Otherwise create the absolute filename from the directory of the .vik file & and the relative filename
+          gchar *full = g_strconcat(dirpath, G_DIR_SEPARATOR_S, line_image, NULL);
+          gchar *absolute = file_realpath_dup ( full ); // resolved into the canonical name
+          vik_waypoint_set_image ( wp, absolute );
+          g_free ( absolute );
+          g_free ( full );
+        }
+      }
 
       if ( line_symbol )
         vik_waypoint_set_symbol ( wp, line_symbol );
@@ -453,7 +465,7 @@ static void gpspoint_process_key_and_value ( const gchar *key, gint key_len, con
   {
     line_altitude = g_ascii_strtod(value, NULL);
   }
-  else if (key_len == 7 && strncasecmp( key, "visible", key_len ) == 0 && value[0] != 'y' && value[0] != 'Y' && value[0] != 't' && value[0] != 'T')
+  else if (key_len == 7 && strncasecmp( key, "visible", key_len ) == 0 && value != NULL && value[0] != 'y' && value[0] != 'Y' && value[0] != 't' && value[0] != 'T')
   {
     line_visible = FALSE;
   }
@@ -497,10 +509,12 @@ static void a_gpspoint_write_waypoint ( const gpointer id, const VikWaypoint *wp
 {
   static struct LatLon ll;
   gchar *s_lat, *s_lon;
-  // Sanity clause
-  if ( wp && !(wp->name) ) {
+  // Sanity clauses
+  if ( !wp )
     return;
-  }
+  if ( !(wp->name) )
+    return;
+
   vik_coord_to_latlon ( &(wp->coord), &ll );
   s_lat = a_coords_dtostr(ll.lat);
   s_lon = a_coords_dtostr(ll.lon);
