@@ -38,9 +38,7 @@ static const char *menu_xml =
 	"      <separator/>"
 	"      <menuitem action='GenImg'/>"
 	"      <menuitem action='GenImgDir'/>"
-#if GTK_CHECK_VERSION(2,10,0)
 	"      <menuitem action='Print'/>"
-#endif
 	"      <separator/>"
 	"      <menuitem action='SaveExit'/>"
 	"      <menuitem action='Exit'/>"
@@ -52,6 +50,7 @@ static const char *menu_xml =
 	"      <menuitem action='Delete'/>"
 	"      <menuitem action='DeleteAll'/>"
 	"      <separator/>"
+	"      <menuitem action='CopyCentre'/>"
 	"      <menuitem action='MapCacheFlush'/>"
 	"      <menuitem action='SetDefaultLocation'/>"
 	"      <menuitem action='Preferences'/>"
@@ -69,6 +68,8 @@ static const char *menu_xml =
 	"      <menuitem action='GotoSearch'/>"
 	"      <menuitem action='GotoLL'/>"
 	"      <menuitem action='GotoUTM'/>"
+	"      <menuitem action='GoBack'/>"
+	"      <menuitem action='GoForward'/>"
 	"      <separator/>"
 	"      <menuitem action='Refresh'/>"
 	"      <separator/>"
@@ -116,34 +117,6 @@ static const char *menu_xml =
 	"      <menuitem action='About'/>"
 	"    </menu>"
 	"  </menubar>"
-	"  <toolbar name='MainToolbar'>"
-	"    <placeholder name='FileToolItems'>"
-	"      <toolitem name='New' action='New'/>"
-	"      <toolitem name='Open' action='Open'/>"
-	"      <toolitem name='Save' action='Save'/>"
-#if GTK_CHECK_VERSION(2,10,0)
-	"      <toolitem name='Print' action='Print'/>"
-#endif
-	"      <toolitem name='Exit' action='Exit'/>"
-	"      <separator/>"
-	"    </placeholder>"
-	"    <placeholder name='UIToolItems'>"
-	"      <toolitem action='FullScreen'/>"
-	"      <toolitem action='ViewSidePanel'/>"
-	"      <separator/>"
-	"    </placeholder>"
-	"    <placeholder name='ViewToolItems'>"
-	"      <toolitem action='GotoDefaultLocation'/>"
-	"      <toolitem action='GotoSearch'/>"
-	"      <separator/>"
-	"    </placeholder>"
-	"    <placeholder name='ToolItems'>"
-	"    <toolitem action='Pan'/>"
-	"    <toolitem action='Zoom'/>"
-	"    <toolitem action='Ruler'/>"
-	"    <toolitem action='Select'/>"
-	"    </placeholder>"
-	"  </toolbar>"
 	"</ui>"
 ;
 
