@@ -25,10 +25,14 @@
 #include <glib.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
+G_BEGIN_DECLS
+
 gboolean a_thumbnails_exists ( const gchar *filename );
 void a_thumbnails_create ( const gchar *filename );
 GdkPixbuf *a_thumbnails_get(const gchar *filename);
 GdkPixbuf *a_thumbnails_get_default ();
 GdkPixbuf *a_thumbnails_scale_pixbuf(GdkPixbuf *src, int max_w, int max_h);
 
+G_END_DECLS
+
 #endif
