@@ -43,13 +43,13 @@
 #include <glib/gi18n.h>
 #ifdef VIK_CONFIG_REALTIME_GPS_TRACKING
 #include <gps.h>
+#include "vikutils.h"
 #endif
 
-#define DISCONNECT_UPDATE_SIGNAL(vl, val) g_signal_handlers_disconnect_matched(vl, G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, val)
 static VikGpsLayer *vik_gps_layer_create (VikViewport *vp);
 static void vik_gps_layer_realize ( VikGpsLayer *val, VikTreeview *vt, GtkTreeIter *layer_iter );
 static void vik_gps_layer_free ( VikGpsLayer *val );
-static void vik_gps_layer_draw ( VikGpsLayer *val, gpointer data );
+static void vik_gps_layer_draw ( VikGpsLayer *val, VikViewport *vp );
 static VikGpsLayer *vik_gps_layer_new ( VikViewport *vp );
 
 static void gps_layer_marshall( VikGpsLayer *val, guint8 **data, gint *len );
@@ -61,7 +61,6 @@ static const gchar* gps_layer_tooltip ( VikGpsLayer *vgl );
 
 static void gps_layer_change_coord_mode ( VikGpsLayer *val, VikCoordMode mode );
 static void gps_layer_add_menu_items( VikGpsLayer *vtl, GtkMenu *menu, gpointer vlp );
-static void gps_layer_drag_drop_request ( VikGpsLayer *val_src, VikGpsLayer *val_dest, GtkTreeIter *src_item_iter, GtkTreePath *dest_path );
 
 static void gps_upload_cb( gpointer layer_and_vlp[2] );
 static void gps_download_cb( gpointer layer_and_vlp[2] );
@@ -104,7 +103,7 @@ typedef struct {
   gint count;
   VikTrwLayer *vtl;
   VikTrack *track;
-  gchar *cmd_args;
+  gchar *babelargs;
   gchar *window_title;
   GtkWidget *dialog;
   GtkWidget *status_label;
@@ -132,6 +131,35 @@ static gchar *params_groups[] = {
 
 enum {GROUP_DATA_MODE, GROUP_REALTIME_MODE};
 
+
+static VikLayerParamData gps_protocol_default ( void )
+{
+  VikLayerParamData data;
+  data.s = g_strdup ( "garmin" );
+  return data;
+}
+
+static VikLayerParamData gps_port_default ( void )
+{
+  VikLayerParamData data;
+  data.s = g_strdup ( "usb:" );
+#ifndef WINDOWS
+  /* Attempt to auto set default USB serial port entry */
+  /* Ordered to make lowest device favourite if available */
+  if (g_access ("/dev/ttyUSB1", R_OK) == 0) {
+	if ( data.s )
+	  g_free ( (gchar *)data.s );
+    data.s = g_strdup ("/dev/ttyUSB1");
+  }
+  if (g_access ("/dev/ttyUSB0", R_OK) == 0) {
+	if ( data.s )
+	  g_free ( (gchar *)data.s );
+    data.s = g_strdup ("/dev/ttyUSB0");
+  }
+#endif
+  return data;
+}
+
 #if defined (VIK_CONFIG_REALTIME_GPS_TRACKING) && defined (GPSD_API_MAJOR_VERSION)
 static gchar *params_vehicle_position[] = {
   N_("Keep vehicle at center"),
@@ -144,24 +172,52 @@ enum {
   VEHICLE_POSITION_ON_SCREEN,
   VEHICLE_POSITION_NONE,
 };
+
+static VikLayerParamData moving_map_method_default ( void ) { return VIK_LPD_UINT ( VEHICLE_POSITION_ON_SCREEN ); }
+
+static VikLayerParamData gpsd_host_default ( void )
+{
+  VikLayerParamData data;
+  data.s = g_strdup ( "localhost" );
+  return data;
+}
+
+static VikLayerParamData gpsd_port_default ( void )
+{
+  VikLayerParamData data;
+  data.s = g_strdup ( DEFAULT_GPSD_PORT );
+  return data;
+}
+
+static VikLayerParamData gpsd_retry_interval_default ( void )
+{
+  VikLayerParamData data;
+  data.s = g_strdup ( "10" );
+  return data;
+}
+
 #endif
 
 static VikLayerParam gps_layer_params[] = {
-  { "gps_protocol", VIK_LAYER_PARAM_STRING, GROUP_DATA_MODE, N_("GPS Protocol:"), VIK_LAYER_WIDGET_COMBOBOX, NULL, NULL, NULL }, // List now assigned at runtime
-  { "gps_port", VIK_LAYER_PARAM_STRING, GROUP_DATA_MODE, N_("Serial Port:"), VIK_LAYER_WIDGET_COMBOBOX, params_ports, NULL, NULL },
-  { "gps_download_tracks", VIK_LAYER_PARAM_BOOLEAN, GROUP_DATA_MODE, N_("Download Tracks:"), VIK_LAYER_WIDGET_CHECKBUTTON, NULL, NULL, NULL },
-  { "gps_upload_tracks", VIK_LAYER_PARAM_BOOLEAN, GROUP_DATA_MODE, N_("Upload Tracks:"), VIK_LAYER_WIDGET_CHECKBUTTON, NULL, NULL, NULL },
-  { "gps_download_routes", VIK_LAYER_PARAM_BOOLEAN, GROUP_DATA_MODE, N_("Download Routes:"), VIK_LAYER_WIDGET_CHECKBUTTON, NULL, NULL, NULL },
-  { "gps_upload_routes", VIK_LAYER_PARAM_BOOLEAN, GROUP_DATA_MODE, N_("Upload Routes:"), VIK_LAYER_WIDGET_CHECKBUTTON, NULL, NULL, NULL },
-  { "gps_download_waypoints", VIK_LAYER_PARAM_BOOLEAN, GROUP_DATA_MODE, N_("Download Waypoints:"), VIK_LAYER_WIDGET_CHECKBUTTON, NULL, NULL, NULL },
-  { "gps_upload_waypoints", VIK_LAYER_PARAM_BOOLEAN, GROUP_DATA_MODE, N_("Upload Waypoints:"), VIK_LAYER_WIDGET_CHECKBUTTON, NULL, NULL, NULL },
+ //	NB gps_layer_inst_init() is performed after parameter registeration
+ //  thus to give the protocols some potential values use the old static list
+ // TODO: find another way to use gps_layer_inst_init()?
+  { VIK_LAYER_GPS, "gps_protocol", VIK_LAYER_PARAM_STRING, GROUP_DATA_MODE, N_("GPS Protocol:"), VIK_LAYER_WIDGET_COMBOBOX, protocols_args, NULL, NULL, gps_protocol_default, NULL, NULL }, // List reassigned at runtime
+  { VIK_LAYER_GPS, "gps_port", VIK_LAYER_PARAM_STRING, GROUP_DATA_MODE, N_("Serial Port:"), VIK_LAYER_WIDGET_COMBOBOX, params_ports, NULL, NULL, gps_port_default, NULL, NULL },
+  { VIK_LAYER_GPS, "gps_download_tracks", VIK_LAYER_PARAM_BOOLEAN, GROUP_DATA_MODE, N_("Download Tracks:"), VIK_LAYER_WIDGET_CHECKBUTTON, NULL, NULL, NULL, vik_lpd_true_default, NULL, NULL },
+  { VIK_LAYER_GPS, "gps_upload_tracks", VIK_LAYER_PARAM_BOOLEAN, GROUP_DATA_MODE, N_("Upload Tracks:"), VIK_LAYER_WIDGET_CHECKBUTTON, NULL, NULL, NULL, vik_lpd_true_default, NULL, NULL },
+  { VIK_LAYER_GPS, "gps_download_routes", VIK_LAYER_PARAM_BOOLEAN, GROUP_DATA_MODE, N_("Download Routes:"), VIK_LAYER_WIDGET_CHECKBUTTON, NULL, NULL, NULL, vik_lpd_true_default, NULL, NULL },
+  { VIK_LAYER_GPS, "gps_upload_routes", VIK_LAYER_PARAM_BOOLEAN, GROUP_DATA_MODE, N_("Upload Routes:"), VIK_LAYER_WIDGET_CHECKBUTTON, NULL, NULL, NULL, vik_lpd_true_default, NULL, NULL },
+  { VIK_LAYER_GPS, "gps_download_waypoints", VIK_LAYER_PARAM_BOOLEAN, GROUP_DATA_MODE, N_("Download Waypoints:"), VIK_LAYER_WIDGET_CHECKBUTTON, NULL, NULL, NULL, vik_lpd_true_default, NULL, NULL },
+  { VIK_LAYER_GPS, "gps_upload_waypoints", VIK_LAYER_PARAM_BOOLEAN, GROUP_DATA_MODE, N_("Upload Waypoints:"), VIK_LAYER_WIDGET_CHECKBUTTON, NULL, NULL, NULL, vik_lpd_true_default, NULL, NULL },
 #if defined (VIK_CONFIG_REALTIME_GPS_TRACKING) && defined (GPSD_API_MAJOR_VERSION)
-  { "record_tracking", VIK_LAYER_PARAM_BOOLEAN, GROUP_REALTIME_MODE, N_("Recording tracks"), VIK_LAYER_WIDGET_CHECKBUTTON, NULL, NULL, NULL },
-  { "center_start_tracking", VIK_LAYER_PARAM_BOOLEAN, GROUP_REALTIME_MODE, N_("Jump to current position on start"), VIK_LAYER_WIDGET_CHECKBUTTON, NULL, NULL, NULL },
-  { "moving_map_method", VIK_LAYER_PARAM_UINT, GROUP_REALTIME_MODE, N_("Moving Map Method:"), VIK_LAYER_WIDGET_RADIOGROUP_STATIC, params_vehicle_position, NULL, NULL },
-  { "gpsd_host", VIK_LAYER_PARAM_STRING, GROUP_REALTIME_MODE, N_("Gpsd Host:"), VIK_LAYER_WIDGET_ENTRY, NULL, NULL, NULL },
-  { "gpsd_port", VIK_LAYER_PARAM_STRING, GROUP_REALTIME_MODE, N_("Gpsd Port:"), VIK_LAYER_WIDGET_ENTRY, NULL, NULL, NULL },
-  { "gpsd_retry_interval", VIK_LAYER_PARAM_STRING, GROUP_REALTIME_MODE, N_("Gpsd Retry Interval (seconds):"), VIK_LAYER_WIDGET_ENTRY, NULL, NULL, NULL },
+  { VIK_LAYER_GPS, "record_tracking", VIK_LAYER_PARAM_BOOLEAN, GROUP_REALTIME_MODE, N_("Recording tracks"), VIK_LAYER_WIDGET_CHECKBUTTON, NULL, NULL, NULL, vik_lpd_true_default, NULL, NULL },
+  { VIK_LAYER_GPS, "center_start_tracking", VIK_LAYER_PARAM_BOOLEAN, GROUP_REALTIME_MODE, N_("Jump to current position on start"), VIK_LAYER_WIDGET_CHECKBUTTON, NULL, NULL, NULL, vik_lpd_false_default, NULL, NULL },
+  { VIK_LAYER_GPS, "moving_map_method", VIK_LAYER_PARAM_UINT, GROUP_REALTIME_MODE, N_("Moving Map Method:"), VIK_LAYER_WIDGET_RADIOGROUP_STATIC, params_vehicle_position, NULL, NULL, moving_map_method_default, NULL, NULL },
+  { VIK_LAYER_GPS, "realtime_update_statusbar", VIK_LAYER_PARAM_BOOLEAN, GROUP_REALTIME_MODE, N_("Update Statusbar:"), VIK_LAYER_WIDGET_CHECKBUTTON, NULL, NULL, N_("Display information in the statusbar on GPS updates"), vik_lpd_true_default, NULL, NULL },
+  { VIK_LAYER_GPS, "gpsd_host", VIK_LAYER_PARAM_STRING, GROUP_REALTIME_MODE, N_("Gpsd Host:"), VIK_LAYER_WIDGET_ENTRY, NULL, NULL, NULL, gpsd_host_default, NULL, NULL },
+  { VIK_LAYER_GPS, "gpsd_port", VIK_LAYER_PARAM_STRING, GROUP_REALTIME_MODE, N_("Gpsd Port:"), VIK_LAYER_WIDGET_ENTRY, NULL, NULL, NULL, gpsd_port_default, NULL, NULL },
+  { VIK_LAYER_GPS, "gpsd_retry_interval", VIK_LAYER_PARAM_STRING, GROUP_REALTIME_MODE, N_("Gpsd Retry Interval (seconds):"), VIK_LAYER_WIDGET_ENTRY, NULL, NULL, NULL, gpsd_retry_interval_default, NULL, NULL },
 #endif /* VIK_CONFIG_REALTIME_GPS_TRACKING */
 };
 enum {
@@ -170,7 +226,13 @@ enum {
   PARAM_DOWNLOAD_ROUTES, PARAM_UPLOAD_ROUTES,
   PARAM_DOWNLOAD_WAYPOINTS, PARAM_UPLOAD_WAYPOINTS,
 #if defined (VIK_CONFIG_REALTIME_GPS_TRACKING) && defined (GPSD_API_MAJOR_VERSION)
-  PARAM_REALTIME_REC, PARAM_REALTIME_CENTER_START, PARAM_VEHICLE_POSITION, PARAM_GPSD_HOST, PARAM_GPSD_PORT, PARAM_GPSD_RETRY_INTERVAL,
+  PARAM_REALTIME_REC,
+  PARAM_REALTIME_CENTER_START,
+  PARAM_VEHICLE_POSITION,
+  PARAM_REALTIME_UPDATE_STATUSBAR,
+  PARAM_GPSD_HOST,
+  PARAM_GPSD_PORT,
+  PARAM_GPSD_RETRY_INTERVAL,
 #endif /* VIK_CONFIG_REALTIME_GPS_TRACKING */
   NUM_PARAMS};
 
@@ -199,6 +261,8 @@ VikLayerInterface vik_gps_layer_interface = {
   (VikLayerFuncDraw)                    vik_gps_layer_draw,
   (VikLayerFuncChangeCoordMode)         gps_layer_change_coord_mode,
 
+  (VikLayerFuncGetTimestamp)            NULL,
+
   (VikLayerFuncSetMenuItemsSelection)   NULL,
   (VikLayerFuncGetMenuItemsSelection)   NULL,
 
@@ -216,6 +280,7 @@ VikLayerInterface vik_gps_layer_interface = {
 
   (VikLayerFuncSetParam)                gps_layer_set_param,
   (VikLayerFuncGetParam)                gps_layer_get_param,
+  (VikLayerFuncChangeParam)             NULL,
 
   (VikLayerFuncReadFileData)            NULL,
   (VikLayerFuncWriteFileData)           NULL,
@@ -225,7 +290,7 @@ VikLayerInterface vik_gps_layer_interface = {
   (VikLayerFuncCopyItem)                NULL,
   (VikLayerFuncPasteItem)               NULL,
   (VikLayerFuncFreeCopiedItem)          NULL,
-  (VikLayerFuncDragDropRequest)		gps_layer_drag_drop_request,
+  (VikLayerFuncDragDropRequest)         NULL,
 
   (VikLayerFuncSelectClick)             NULL,
   (VikLayerFuncSelectMove)              NULL,
@@ -288,6 +353,9 @@ struct _VikGpsLayer {
   gboolean realtime_record;
   gboolean realtime_jump_to_start;
   guint vehicle_position;
+  gboolean realtime_update_statusbar;
+  VikTrackpoint *trkpt;
+  VikTrackpoint *trkpt_prev;
 #endif /* VIK_CONFIG_REALTIME_GPS_TRACKING */
   gchar *protocol;
   gchar *serial_port;
@@ -306,7 +374,7 @@ static void gps_layer_inst_init ( VikGpsLayer *self )
 {
   gint new_proto = 0;
   // +1 for luck (i.e the NULL terminator)
-  gchar **new_protocols = g_malloc(1 + g_list_length(a_babel_device_list)*sizeof(gpointer));
+  gchar **new_protocols = g_malloc_n(1 + g_list_length(a_babel_device_list), sizeof(gpointer));
 
   GList *gl = g_list_first ( a_babel_device_list );
   while ( gl ) {
@@ -317,8 +385,7 @@ static void gps_layer_inst_init ( VikGpsLayer *self )
   }
   new_protocols[new_proto] = NULL;
 
-  vik_gps_layer_interface.params[0].widget_data = new_protocols;
-  // assigned to [0] because this^ is the GPS protocol in the params list
+  vik_gps_layer_interface.params[PARAM_PROTOCOL].widget_data = new_protocols;
 }
 
 GType vik_gps_layer_get_type ()
@@ -353,7 +420,7 @@ static VikGpsLayer *vik_gps_layer_create (VikViewport *vp)
   vik_layer_rename ( VIK_LAYER(rv), vik_gps_layer_interface.name );
 
   for (i = 0; i < NUM_TRW; i++) {
-    rv->trw_children[i] = VIK_TRW_LAYER(vik_layer_create ( VIK_LAYER_TRW, vp, NULL, FALSE ));
+    rv->trw_children[i] = VIK_TRW_LAYER(vik_layer_create ( VIK_LAYER_TRW, vp, FALSE ));
     vik_layer_set_menu_items_selection(VIK_LAYER(rv->trw_children[i]), VIK_MENU_ITEM_ALL & ~(VIK_MENU_ITEM_CUT|VIK_MENU_ITEM_DELETE));
   }
   return rv;
@@ -417,7 +484,8 @@ static VikGpsLayer *gps_layer_unmarshall( guint8 *data, gint len, VikViewport *v
     child_layer = vik_layer_unmarshall ( data + sizeof(gint), alm_size, vvp );
     if (child_layer) {
       rv->trw_children[i++] = (VikTrwLayer *)child_layer;
-      g_signal_connect_swapped ( G_OBJECT(child_layer), "update", G_CALLBACK(vik_layer_emit_update_secondary), rv );
+      // NB no need to attach signal update handler here
+      //  as this will always be performed later on in vik_gps_layer_realize()
     }
     alm_next;
   }
@@ -488,14 +556,18 @@ static gboolean gps_layer_set_param ( VikGpsLayer *vgl, guint16 id, VikLayerPara
       break;
 #if defined (VIK_CONFIG_REALTIME_GPS_TRACKING) && defined (GPSD_API_MAJOR_VERSION)
     case PARAM_GPSD_HOST:
-      if (vgl->gpsd_host)
-        g_free(vgl->gpsd_host);
-      vgl->gpsd_host = g_strdup(data.s);
+      if (data.s) {
+        if (vgl->gpsd_host)
+          g_free(vgl->gpsd_host);
+        vgl->gpsd_host = g_strdup(data.s);
+      }
       break;
     case PARAM_GPSD_PORT:
-      if (vgl->gpsd_port)
-        g_free(vgl->gpsd_port);
-      vgl->gpsd_port = g_strdup(data.s); /* TODO: check for number */
+      if (data.s) {
+        if (vgl->gpsd_port)
+          g_free(vgl->gpsd_port);
+        vgl->gpsd_port = g_strdup(data.s);
+      }
       break;
     case PARAM_GPSD_RETRY_INTERVAL:
       vgl->gpsd_retry_interval = strtol(data.s, NULL, 10);
@@ -509,6 +581,9 @@ static gboolean gps_layer_set_param ( VikGpsLayer *vgl, guint16 id, VikLayerPara
     case PARAM_VEHICLE_POSITION:
       vgl->vehicle_position = data.u;
       break;
+    case PARAM_REALTIME_UPDATE_STATUSBAR:
+      vgl->realtime_update_statusbar = data.b;
+      break;
 #endif /* VIK_CONFIG_REALTIME_GPS_TRACKING */
     default:
       g_warning("gps_layer_set_param(): unknown parameter");
@@ -567,6 +642,9 @@ static VikLayerParamData gps_layer_get_param ( VikGpsLayer *vgl, guint16 id, gbo
     case PARAM_VEHICLE_POSITION:
       rv.u = vgl->vehicle_position;
       break;
+    case PARAM_REALTIME_UPDATE_STATUSBAR:
+      rv.u = vgl->realtime_update_statusbar;
+      break;
 #endif /* VIK_CONFIG_REALTIME_GPS_TRACKING */
     default:
       g_warning(_("%s: unknown parameter"), __FUNCTION__);
@@ -587,85 +665,59 @@ VikGpsLayer *vik_gps_layer_new (VikViewport *vp)
   vgl->cur_read_child = 0;
 
 #if defined (VIK_CONFIG_REALTIME_GPS_TRACKING) && defined (GPSD_API_MAJOR_VERSION)
-  vgl->realtime_tracking = FALSE;
   vgl->first_realtime_trackpoint = FALSE;
+  vgl->trkpt = NULL;
+  vgl->trkpt_prev = NULL;
   vgl->vgpsd = NULL;
   vgl->realtime_io_channel = NULL;
   vgl->realtime_io_watch_id = 0;
   vgl->realtime_retry_timer = 0;
-  vgl->realtime_track_gc = vik_viewport_new_gc ( vp, "#203070", 2 );
-  vgl->realtime_track_bg_gc = vik_viewport_new_gc ( vp, "grey", 2 );
-  vgl->realtime_track_pt1_gc = vik_viewport_new_gc ( vp, "red", 2 );
-  vgl->realtime_track_pt2_gc = vik_viewport_new_gc ( vp, "green", 2 );
-  vgl->realtime_track_pt_gc = vgl->realtime_track_pt1_gc;
+  if ( vp ) {
+    vgl->realtime_track_gc = vik_viewport_new_gc ( vp, "#203070", 2 );
+    vgl->realtime_track_bg_gc = vik_viewport_new_gc ( vp, "grey", 2 );
+    vgl->realtime_track_pt1_gc = vik_viewport_new_gc ( vp, "red", 2 );
+    vgl->realtime_track_pt2_gc = vik_viewport_new_gc ( vp, "green", 2 );
+    vgl->realtime_track_pt_gc = vgl->realtime_track_pt1_gc;
+  }
   vgl->realtime_track = NULL;
+#endif // VIK_CONFIG_REALTIME_GPS_TRACKING
 
-  /* Setting params here */
-  vgl->gpsd_host = g_strdup("localhost");
-  vgl->gpsd_port = g_strdup(DEFAULT_GPSD_PORT);
-  vgl->realtime_record = TRUE;
-  vgl->realtime_jump_to_start = TRUE;
-  vgl->vehicle_position = VEHICLE_POSITION_ON_SCREEN;
-  vgl->gpsd_retry_interval = 10;
-#endif /* VIK_CONFIG_REALTIME_GPS_TRACKING */
-  vgl->protocol = g_strdup("garmin");
-  vgl->serial_port = NULL;
-  vgl->download_tracks = TRUE;
-  vgl->download_waypoints = TRUE;
-  vgl->download_routes = TRUE;
-  vgl->upload_routes = TRUE;
-  vgl->upload_tracks = TRUE;
-  vgl->upload_waypoints = TRUE;
-
-#ifndef WINDOWS
-  /* Attempt to auto set default USB serial port entry */
-  /* Ordered to make lowest device favourite if available */
-  if (g_access ("/dev/ttyUSB1", R_OK) == 0) {
-    if (vgl->serial_port != NULL)
-      g_free (vgl->serial_port);
-    vgl->serial_port = g_strdup ("/dev/ttyUSB1");
-  }
-  if (g_access ("/dev/ttyUSB0", R_OK) == 0) {
-    if (vgl->serial_port != NULL)
-      g_free (vgl->serial_port);
-    vgl->serial_port = g_strdup ("/dev/ttyUSB0");
-  }
-#endif
+  vik_layer_set_defaults ( VIK_LAYER(vgl), vp );
 
   return vgl;
 }
 
-static void vik_gps_layer_draw ( VikGpsLayer *vgl, gpointer data )
+static void vik_gps_layer_draw ( VikGpsLayer *vgl, VikViewport *vp )
 {
   gint i;
   VikLayer *vl;
-  VikLayer *trigger = VIK_LAYER(vik_viewport_get_trigger( VIK_VIEWPORT(data) ));
+  VikLayer *trigger = VIK_LAYER(vik_viewport_get_trigger( vp ));
 
   for (i = 0; i < NUM_TRW; i++) {
     vl = VIK_LAYER(vgl->trw_children[i]);
     if (vl == trigger) {
-      if ( vik_viewport_get_half_drawn ( VIK_VIEWPORT(data) ) ) {
-        vik_viewport_set_half_drawn ( VIK_VIEWPORT(data), FALSE );
-        vik_viewport_snapshot_load( VIK_VIEWPORT(data) );
+      if ( vik_viewport_get_half_drawn ( vp ) ) {
+        vik_viewport_set_half_drawn ( vp, FALSE );
+        vik_viewport_snapshot_load( vp );
       } else {
-        vik_viewport_snapshot_save( VIK_VIEWPORT(data) );
+        vik_viewport_snapshot_save( vp );
       }
     }
-    if (!vik_viewport_get_half_drawn( VIK_VIEWPORT(data)))
-      vik_layer_draw ( vl, data );
+    if (!vik_viewport_get_half_drawn(vp))
+      vik_layer_draw ( vl, vp );
   }
 #if defined (VIK_CONFIG_REALTIME_GPS_TRACKING) && defined (GPSD_API_MAJOR_VERSION)
   if (vgl->realtime_tracking) {
     if (VIK_LAYER(vgl) == trigger) {
-      if ( vik_viewport_get_half_drawn ( VIK_VIEWPORT(data) ) ) {
-        vik_viewport_set_half_drawn ( VIK_VIEWPORT(data), FALSE );
-        vik_viewport_snapshot_load( VIK_VIEWPORT(data) );
+      if ( vik_viewport_get_half_drawn ( vp ) ) {
+        vik_viewport_set_half_drawn ( vp, FALSE );
+        vik_viewport_snapshot_load( vp );
       } else {
-        vik_viewport_snapshot_save( VIK_VIEWPORT(data) );
+        vik_viewport_snapshot_save( vp );
       }
     }
-    if (!vik_viewport_get_half_drawn( VIK_VIEWPORT(data)))
-      realtime_tracking_draw(vgl, VIK_VIEWPORT(data));
+    if (!vik_viewport_get_half_drawn(vp))
+      realtime_tracking_draw(vgl, vp);
   }
 #endif /* VIK_CONFIG_REALTIME_GPS_TRACKING */
 }
@@ -746,14 +798,9 @@ static void gps_layer_add_menu_items( VikGpsLayer *vgl, GtkMenu *menu, gpointer
 
 static void disconnect_layer_signal ( VikLayer *vl, VikGpsLayer *vgl )
 {
-  guint number_handlers = DISCONNECT_UPDATE_SIGNAL(vl,vgl);
+  guint number_handlers = g_signal_handlers_disconnect_matched(vl, G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, vgl);
   if ( number_handlers != 1 ) {
-    /*
-      NB It's not fatal if this gives 2 for example! Hence removal of the g_assert
-      This happens when copied GPS layer is deleted (not sure why the number_handlers would be 2)
-      I don't think there's any side effects and certainly better than the program just aborting
-    */
-    g_warning(_("Unexpected number of disconnected handlers: %d"), number_handlers);
+    g_critical(_("Unexpected number of disconnected handlers: %d"), number_handlers);
   }
 }
 
@@ -778,23 +825,6 @@ static void vik_gps_layer_free ( VikGpsLayer *vgl )
 #endif /* VIK_CONFIG_REALTIME_GPS_TRACKING */
 }
 
-gboolean vik_gps_layer_delete ( VikGpsLayer *vgl, GtkTreeIter *iter )
-{
-  gint i;
-  VikLayer *l = VIK_LAYER( vik_treeview_item_get_pointer ( VIK_LAYER(vgl)->vt, iter ) );
-  gboolean was_visible = l->visible;
-
-  vik_treeview_item_delete ( VIK_LAYER(vgl)->vt, iter );
-  for (i = 0; i < NUM_TRW; i++) {
-    if (VIK_LAYER(vgl->trw_children[i]) == l)
-      vgl->trw_children[i] = NULL;
-  }
-  g_assert(DISCONNECT_UPDATE_SIGNAL(l,vgl)==1);
-  g_object_unref ( l );
-
-  return was_visible;
-}
-
 static void vik_gps_layer_realize ( VikGpsLayer *vgl, VikTreeview *vt, GtkTreeIter *layer_iter )
 {
   GtkTreeIter iter;
@@ -808,8 +838,8 @@ static void vik_gps_layer_realize ( VikGpsLayer *vgl, VikTreeview *vt, GtkTreeIt
   for (ix = 0; ix < NUM_TRW; ix++) {
     VikLayer * trw = VIK_LAYER(vgl->trw_children[ix]);
     vik_treeview_add_layer ( VIK_LAYER(vgl)->vt, layer_iter, &iter,
-        _(trw_names[ix]), vgl, 
-        trw, trw->type, trw->type );
+        _(trw_names[ix]), vgl, TRUE,
+        trw, trw->type, trw->type, vik_layer_get_timestamp(trw) );
     if ( ! trw->visible )
       vik_treeview_item_set_visible ( VIK_LAYER(vgl)->vt, &iter, FALSE );
     vik_layer_realize ( trw, VIK_LAYER(vgl)->vt, &iter );
@@ -845,30 +875,11 @@ gboolean vik_gps_layer_is_empty ( VikGpsLayer *vgl )
   return TRUE;
 }
 
-static void gps_layer_drag_drop_request ( VikGpsLayer *val_src, VikGpsLayer *val_dest, GtkTreeIter *src_item_iter, GtkTreePath *dest_path )
-{
-  VikTreeview *vt = VIK_LAYER(val_src)->vt;
-  VikLayer *vl = vik_treeview_item_get_pointer(vt, src_item_iter);
-  gchar *dp;
-
-  dp = gtk_tree_path_to_string(dest_path);
-
-  /* vik_gps_layer_delete unrefs, but we don't want that here.
-   * we're still using the layer. */
-  g_object_ref ( vl );
-  vik_gps_layer_delete(val_src, src_item_iter);
-
-  g_free(dp);
-}
-
 static void gps_session_delete(GpsSession *sess)
 {
-  /* TODO */
-  g_mutex_free(sess->mutex);
-  g_free(sess->cmd_args);
-
+  vik_mutex_free(sess->mutex);
+  g_free(sess->babelargs);
   g_free(sess);
-
 }
 
 static void set_total_count(gint cnt, GpsSession *sess)
@@ -887,8 +898,9 @@ static void set_total_count(gint cnt, GpsSession *sess)
         {
           // Maybe a gpsbabel bug/feature (upto at least v1.4.3 or maybe my Garmin device) but the count always seems x2 too many for routepoints
           gint mycnt = (cnt / 2) + 1;
-          tmp_str = ngettext("Downloading %d routepoint...", "Downloading %d routepoints...", mycnt); break;
+          tmp_str = ngettext("Downloading %d routepoint...", "Downloading %d routepoints...", mycnt);
           sess->total_count = mycnt;
+          break;
         }
       }
     }
@@ -1168,11 +1180,12 @@ static void gps_comm_thread(GpsSession *sess)
 {
   gboolean result;
 
-  if (sess->direction == GPS_DOWN)
-    result = a_babel_convert_from (sess->vtl, sess->cmd_args, sess->port,
-        (BabelStatusFunc) gps_download_progress_func, sess, NULL);
+  if (sess->direction == GPS_DOWN) {
+    ProcessOptions po = { sess->babelargs, sess->port, NULL, NULL, NULL, NULL };
+    result = a_babel_convert_from (sess->vtl, &po, (BabelStatusFunc) gps_download_progress_func, sess, NULL);
+  }
   else {
-    result = a_babel_convert_to (sess->vtl, sess->track, sess->cmd_args, sess->port,
+    result = a_babel_convert_to (sess->vtl, sess->track, sess->babelargs, sess->port,
         (BabelStatusFunc) gps_upload_progress_func, sess);
   }
 
@@ -1191,11 +1204,12 @@ static void gps_comm_thread(GpsSession *sess)
       if (!sess->realtime_tracking)
 #endif
       {
-	if ( sess->vvp && sess->direction == GPS_DOWN ) {
-	  /* View the data available */
-	  vik_trw_layer_auto_set_view ( sess->vtl, sess->vvp) ;
-	  vik_layer_emit_update ( VIK_LAYER(sess->vtl) ); // NB update from background thread
-	}
+        if ( sess->vvp && sess->direction == GPS_DOWN ) {
+          vik_layer_post_read ( VIK_LAYER(sess->vtl), sess->vvp, TRUE );
+          /* View the data available */
+          vik_trw_layer_auto_set_view ( sess->vtl, sess->vvp ) ;
+          vik_layer_emit_update ( VIK_LAYER(sess->vtl) ); // NB update from background thread
+        }
       }
     } else {
       /* canceled */
@@ -1249,7 +1263,7 @@ gint vik_gps_comm ( VikTrwLayer *vtl,
   char *routes = NULL;
   char *waypoints = NULL;
 
-  sess->mutex = g_mutex_new();
+  sess->mutex = vik_mutex_new();
   sess->direction = dir;
   sess->vtl = vtl;
   sess->track = track;
@@ -1286,7 +1300,7 @@ gint vik_gps_comm ( VikTrwLayer *vtl,
   else
     waypoints = "";
 
-  sess->cmd_args = g_strdup_printf("-D 9 %s %s %s -%c %s",
+  sess->babelargs = g_strdup_printf("-D 9 %s %s %s -%c %s",
 				   tracks, routes, waypoints, (dir == GPS_DOWN) ? 'i' : 'o', protocol);
   tracks = NULL;
   waypoints = NULL;
@@ -1299,7 +1313,7 @@ gint vik_gps_comm ( VikTrwLayer *vtl,
     gtk_window_set_title ( GTK_WINDOW(sess->dialog), sess->window_title );
 
     sess->status_label = gtk_label_new (_("Status: detecting gpsbabel"));
-    gtk_box_pack_start ( GTK_BOX(GTK_DIALOG(sess->dialog)->vbox), sess->status_label, FALSE, FALSE, 5 );
+    gtk_box_pack_start ( GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(sess->dialog))), sess->status_label, FALSE, FALSE, 5 );
     gtk_widget_show_all(sess->status_label);
 
     sess->gps_label = gtk_label_new (_("GPS device: N/A"));
@@ -1309,10 +1323,10 @@ gint vik_gps_comm ( VikTrwLayer *vtl,
     sess->trk_label = gtk_label_new ("");
     sess->rte_label = gtk_label_new ("");
 
-    gtk_box_pack_start ( GTK_BOX(GTK_DIALOG(sess->dialog)->vbox), sess->gps_label, FALSE, FALSE, 5 );
-    gtk_box_pack_start ( GTK_BOX(GTK_DIALOG(sess->dialog)->vbox), sess->wp_label, FALSE, FALSE, 5 );
-    gtk_box_pack_start ( GTK_BOX(GTK_DIALOG(sess->dialog)->vbox), sess->trk_label, FALSE, FALSE, 5 );
-    gtk_box_pack_start ( GTK_BOX(GTK_DIALOG(sess->dialog)->vbox), sess->rte_label, FALSE, FALSE, 5 );
+    gtk_box_pack_start ( GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(sess->dialog))), sess->gps_label, FALSE, FALSE, 5 );
+    gtk_box_pack_start ( GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(sess->dialog))), sess->wp_label, FALSE, FALSE, 5 );
+    gtk_box_pack_start ( GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(sess->dialog))), sess->trk_label, FALSE, FALSE, 5 );
+    gtk_box_pack_start ( GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(sess->dialog))), sess->rte_label, FALSE, FALSE, 5 );
 
     gtk_widget_show_all(sess->dialog);
 
@@ -1320,7 +1334,11 @@ gint vik_gps_comm ( VikTrwLayer *vtl,
     sess->total_count = -1;
 
     // Starting gps read/write thread
-    g_thread_create((GThreadFunc)gps_comm_thread, sess, FALSE, NULL );
+#if GLIB_CHECK_VERSION (2, 32, 0)
+    g_thread_try_new ( "gps_comm_thread", (GThreadFunc)gps_comm_thread, sess, NULL );
+#else
+    g_thread_create ( (GThreadFunc)gps_comm_thread, sess, FALSE, NULL );
+#endif
 
     gtk_dialog_set_default_response ( GTK_DIALOG(sess->dialog), GTK_RESPONSE_ACCEPT );
     gtk_dialog_run(GTK_DIALOG(sess->dialog));
@@ -1342,7 +1360,8 @@ gint vik_gps_comm ( VikTrwLayer *vtl,
     if ( turn_off ) {
       // No need for thread for powering off device (should be quick operation...) - so use babel command directly:
       gchar *device_off = g_strdup_printf("-i %s,%s", protocol, "power_off");
-      gboolean result = a_babel_convert_from (NULL, (const char*)device_off, (const char*)port, NULL, NULL, NULL);
+      ProcessOptions po = { device_off, port, NULL, NULL, NULL, NULL };
+      gboolean result = a_babel_convert_from (NULL, &po, NULL, NULL, NULL);
       if ( !result )
         a_dialog_error_msg ( VIK_GTK_WINDOW_FROM_LAYER(vtl), _("Could not turn off device.") );
       g_free ( device_off );
@@ -1450,7 +1469,8 @@ static void realtime_tracking_draw(VikGpsLayer *vgl, VikViewport *vp)
   if ( vgl->realtime_fix.fix.latitude > lse.lat &&
        vgl->realtime_fix.fix.latitude < lnw.lat &&
        vgl->realtime_fix.fix.longitude > lnw.lon &&
-       vgl->realtime_fix.fix.longitude < lse.lon ) {
+       vgl->realtime_fix.fix.longitude < lse.lon &&
+       !isnan (vgl->realtime_fix.fix.track) ) {
     VikCoord gps;
     gint x, y;
     gint half_back_x, half_back_y;
@@ -1500,7 +1520,7 @@ static void realtime_tracking_draw(VikGpsLayer *vgl, VikViewport *vp)
   }
 }
 
-static void create_realtime_trackpoint(VikGpsLayer *vgl, gboolean forced)
+static VikTrackpoint* create_realtime_trackpoint(VikGpsLayer *vgl, gboolean forced)
 {
     struct LatLon ll;
     GList *last_tp;
@@ -1511,13 +1531,13 @@ static void create_realtime_trackpoint(VikGpsLayer *vgl, gboolean forced)
     time_t last_timestamp = vgl->last_fix.fix.time;
 
     if (cur_timestamp < last_timestamp) {
-      return;
+      return NULL;
     }
 
     if (vgl->realtime_record && vgl->realtime_fix.dirty) {
       gboolean replace = FALSE;
-      int heading = (int)floor(vgl->realtime_fix.fix.track);
-      int last_heading = (int)floor(vgl->last_fix.fix.track);
+      int heading = isnan(vgl->realtime_fix.fix.track) ? 0 : (int)floor(vgl->realtime_fix.fix.track);
+      int last_heading = isnan(vgl->last_fix.fix.track) ? 0 : (int)floor(vgl->last_fix.fix.track);
       int alt = isnan(vgl->realtime_fix.fix.altitude) ? VIK_DEFAULT_ALTITUDE : floor(vgl->realtime_fix.fix.altitude);
       int last_alt = isnan(vgl->last_fix.fix.altitude) ? VIK_DEFAULT_ALTITUDE : floor(vgl->last_fix.fix.altitude);
       if (((last_tp = g_list_last(vgl->realtime_track->trackpoints)) != NULL) &&
@@ -1551,12 +1571,34 @@ static void create_realtime_trackpoint(VikGpsLayer *vgl, gboolean forced)
         vik_coord_load_from_latlon(&tp->coord,
              vik_trw_layer_get_coord_mode(vgl->trw_children[TRW_REALTIME]), &ll);
 
-        vgl->realtime_track->trackpoints = g_list_append(vgl->realtime_track->trackpoints, tp);
+        vik_track_add_trackpoint ( vgl->realtime_track, tp, TRUE ); // Ensure bounds is recalculated
         vgl->realtime_fix.dirty = FALSE;
         vgl->realtime_fix.satellites_used = 0;
         vgl->last_fix = vgl->realtime_fix;
+        return tp;
       }
     }
+    return NULL;
+}
+
+#define VIK_SETTINGS_GPS_STATUSBAR_FORMAT "gps_statusbar_format"
+
+static void update_statusbar ( VikGpsLayer *vgl, VikWindow *vw )
+{
+  gchar *statusbar_format_code = NULL;
+  gboolean need2free = FALSE;
+  if ( !a_settings_get_string ( VIK_SETTINGS_GPS_STATUSBAR_FORMAT, &statusbar_format_code ) ) {
+    // Otherwise use default
+    statusbar_format_code = g_strdup ( "GSA" );
+    need2free = TRUE;
+  }
+
+  gchar *msg = vu_trackpoint_formatted_message ( statusbar_format_code, vgl->trkpt, vgl->trkpt_prev, vgl->realtime_track, vgl->last_fix.fix.climb );
+  vik_statusbar_set_message ( vik_window_get_statusbar (vw), VIK_STATUSBAR_INFO, msg );
+  g_free ( msg );
+
+  if ( need2free )
+    g_free ( statusbar_format_code );
 
 }
 
@@ -1572,8 +1614,7 @@ static void gpsd_raw_hook(VglGpsd *vgpsd, gchar *data)
 
   if ((vgpsd->gpsd.fix.mode >= MODE_2D) &&
       !isnan(vgpsd->gpsd.fix.latitude) &&
-      !isnan(vgpsd->gpsd.fix.longitude) &&
-      !isnan(vgpsd->gpsd.fix.track)) {
+      !isnan(vgpsd->gpsd.fix.longitude)) {
 
     VikWindow *vw = VIK_WINDOW(VIK_GTK_WINDOW_FROM_LAYER(vgl));
     VikViewport *vvp = vik_window_viewport(vw);
@@ -1591,7 +1632,7 @@ static void gpsd_raw_hook(VglGpsd *vgpsd, gchar *data)
 
     if ((vgl->vehicle_position == VEHICLE_POSITION_CENTERED) ||
         (vgl->realtime_jump_to_start && vgl->first_realtime_trackpoint)) {
-      vik_viewport_set_center_coord(vvp, &vehicle_coord);
+      vik_viewport_set_center_coord(vvp, &vehicle_coord, FALSE);
       update_all = TRUE;
     }
     else if (vgl->vehicle_position == VEHICLE_POSITION_ON_SCREEN) {
@@ -1617,7 +1658,14 @@ static void gpsd_raw_hook(VglGpsd *vgpsd, gchar *data)
     }
 
     vgl->first_realtime_trackpoint = FALSE;
-    create_realtime_trackpoint(vgl, FALSE);
+
+    vgl->trkpt = create_realtime_trackpoint ( vgl, FALSE );
+
+    if ( vgl->trkpt ) {
+      if ( vgl->realtime_update_statusbar )
+	update_statusbar ( vgl, vw );
+      vgl->trkpt_prev = vgl->trkpt;
+    }
 
     vik_layer_emit_update ( update_all ? VIK_LAYER(vgl) : VIK_LAYER(vgl->trw_children[TRW_REALTIME]) ); // NB update from background thread
   }
@@ -1629,7 +1677,7 @@ static gboolean gpsd_data_available(GIOChannel *source, GIOCondition condition,
   if (condition == G_IO_IN) {
 #if GPSD_API_MAJOR_VERSION == 3 || GPSD_API_MAJOR_VERSION == 4
     if (!gps_poll(&vgl->vgpsd->gpsd)) {
-#elif GPSD_API_MAJOR_VERSION == 5
+#elif GPSD_API_MAJOR_VERSION == 5 || GPSD_API_MAJOR_VERSION == 6
     if (gps_read(&vgl->vgpsd->gpsd) > -1) {
       // Reuse old function to perform operations on the new GPS data
       gpsd_raw_hook(vgl->vgpsd, NULL);
@@ -1674,7 +1722,7 @@ static gboolean rt_gpsd_try_connect(gpointer *data)
   vgl->vgpsd = g_malloc(sizeof(VglGpsd));
 
   if (gps_open_r(vgl->gpsd_host, vgl->gpsd_port, /*(struct gps_data_t *)*/vgl->vgpsd) != 0) {
-#elif GPSD_API_MAJOR_VERSION == 5
+#elif GPSD_API_MAJOR_VERSION == 5 || GPSD_API_MAJOR_VERSION == 6
   vgl->vgpsd = g_malloc(sizeof(VglGpsd));
   if (gps_open(vgl->gpsd_host, vgl->gpsd_port, &vgl->vgpsd->gpsd) != 0) {
 #else
@@ -1713,7 +1761,7 @@ static gboolean rt_gpsd_try_connect(gpointer *data)
 #if GPSD_API_MAJOR_VERSION == 3
   gps_query(&vgl->vgpsd->gpsd, "w+x");
 #endif
-#if GPSD_API_MAJOR_VERSION == 4 || GPSD_API_MAJOR_VERSION == 5
+#if GPSD_API_MAJOR_VERSION == 4 || GPSD_API_MAJOR_VERSION == 5 || GPSD_API_MAJOR_VERSION == 6
   gps_stream(&vgl->vgpsd->gpsd, WATCH_ENABLE, NULL);
 #endif
 
@@ -1769,13 +1817,13 @@ static void rt_gpsd_disconnect(VikGpsLayer *vgl)
     vgl->realtime_io_channel = NULL;
   }
   if (vgl->vgpsd) {
-#if GPSD_API_MAJOR_VERSION == 4 || GPSD_API_MAJOR_VERSION == 5
+#if GPSD_API_MAJOR_VERSION == 4 || GPSD_API_MAJOR_VERSION == 5 || GPSD_API_MAJOR_VERSION == 6
     gps_stream(&vgl->vgpsd->gpsd, WATCH_DISABLE, NULL);
 #endif
     gps_close(&vgl->vgpsd->gpsd);
 #if GPSD_API_MAJOR_VERSION == 3
     free(vgl->vgpsd);
-#elif GPSD_API_MAJOR_VERSION == 4 || GPSD_API_MAJOR_VERSION == 5
+#elif GPSD_API_MAJOR_VERSION == 4 || GPSD_API_MAJOR_VERSION == 5 || GPSD_API_MAJOR_VERSION == 6
     g_free(vgl->vgpsd);
 #endif
     vgl->vgpsd = NULL;
@@ -1801,10 +1849,12 @@ static void gps_start_stop_tracking_cb( gpointer layer_and_vlp[2])
     if (!rt_gpsd_connect(vgl, TRUE)) {
       vgl->first_realtime_trackpoint = FALSE;
       vgl->realtime_tracking = FALSE;
+      vgl->trkpt = NULL;
     }
   }
   else {  /* stop realtime tracking */
     vgl->first_realtime_trackpoint = FALSE;
+    vgl->trkpt = NULL;
     rt_gpsd_disconnect(vgl);
   }
 }
